import { Module } from 'vuex'
import { RootState } from '@/store/store'
import { getStoreBuilder } from 'vuex-typex'
import { FeedbackStageEnum } from '@/models'
import { FeedbackLabels } from '../feedback-labels'

export const namespace = 'feedbackSearchOptions'

export interface FeedbackSearchOptionsState {
  showFinal: boolean
  searchOtherUserComments: boolean
  caseSensitive: boolean
  useRegex: boolean
  filterByTutors: string[]
  filterByStage: string,
  filterByLabels: string[],
  filterByExcludingLabels: string[]
}

function initialState (): FeedbackSearchOptionsState {
  return {
    showFinal: true,
    searchOtherUserComments: false,
    caseSensitive: false,
    useRegex: false,
    filterByTutors: [],
    filterByStage: '',
    filterByLabels: [],
    filterByExcludingLabels: []
  }
}

const mb = getStoreBuilder<RootState>().module('FeedbackSearchOptions', initialState())

const mapStageDisplayToApiString: {[index: string]: string} = {
  'Initial feedback': FeedbackStageEnum.Creation,
  'Validation': FeedbackStageEnum.Validation
}

const stateGetter = mb.state()

const stageFilterStringGetter = mb.read(function stageFilterString (state: FeedbackSearchOptionsState) {
  return mapStageDisplayToApiString[state.filterByStage] || 'all'
})

const labelFilterMappedGetter = mb.read(function labelFilterMapped (state: FeedbackSearchOptionsState) {
  return state.filterByLabels.map(labelName => {
    return FeedbackLabels.state.labels.find(label => label.name === labelName)
  })
})

const labelExcludeFilterMappedGetter = mb.read(function labelExcludeFilterMapped (state: FeedbackSearchOptionsState) {
  return state.filterByExcludingLabels.map(labelName => {
    return FeedbackLabels.state.labels.find(label => label.name === labelName)
  })
})

function SET_SHOW_FINAL (state: FeedbackSearchOptionsState, val: boolean) {
  state.showFinal = val
}
function SET_SEARCH_OTHER_USER_COMMENTS (state: FeedbackSearchOptionsState, val: boolean) {
  state.searchOtherUserComments = val
}
function SET_CASE_SENSITIVE (state: FeedbackSearchOptionsState, val: boolean) {
  state.caseSensitive = val
}
function SET_USE_REGEX (state: FeedbackSearchOptionsState, val: boolean) {
  state.useRegex = val
}
function SET_FILTER_BY_TUTORS (state: FeedbackSearchOptionsState, val: string[]) {
  state.filterByTutors = val
}
function SET_FILTER_BY_STAGE (state: FeedbackSearchOptionsState, val: string) {
  state.filterByStage = val
}
function SET_FILTER_BY_LABELS (state: FeedbackSearchOptionsState, val: string[]) {
  state.filterByLabels = val
}
function SET_FILTER_BY_EXCLUDING_LABELS (state: FeedbackSearchOptionsState, val: string[]) {
  state.filterByExcludingLabels = val
}

export const FeedbackSearchOptions = {
  get state () { return stateGetter() },
  get stageFilterString () { return stageFilterStringGetter() },
  get labelFilterMapped () { return labelFilterMappedGetter() },
  get labelExcludeFilterMapped () { return labelExcludeFilterMappedGetter() },

  SET_SHOW_FINAL: mb.commit(SET_SHOW_FINAL),
  SET_SEARCH_OTHER_USER_COMMENTS: mb.commit(SET_SEARCH_OTHER_USER_COMMENTS),
  SET_CASE_SENSITIVE: mb.commit(SET_CASE_SENSITIVE),
  SET_USE_REGEX: mb.commit(SET_USE_REGEX),
  SET_FILTER_BY_TUTORS: mb.commit(SET_FILTER_BY_TUTORS),
  SET_FILTER_BY_STAGE: mb.commit(SET_FILTER_BY_STAGE),
  SET_FILTER_BY_LABELS: mb.commit(SET_FILTER_BY_LABELS),
  SET_FILTER_BY_EXCLUDING_LABELS: mb.commit(SET_FILTER_BY_EXCLUDING_LABELS)
}
